@@ -5,22 +5,28 @@
 
     A colorful style, inspired by CodeRay.
 
-    :copyright: 2006 by Georg Brandl.
-    :license: BSD, see LICENSE for more details.
+    :copyright: Copyright 2006-2014 by the Pygments team, see AUTHORS.
+    :license: BSD, see LICENSE for details.
 """
 
 from pygments.style import Style
 from pygments.token import Keyword, Name, Comment, String, Error, \
-     Number, Operator, Generic
+     Number, Operator, Generic, Whitespace
 
 
 class ColorfulStyle(Style):
+    """
+    A colorful style, inspired by CodeRay.
+    """
 
     default_style = ""
 
     styles = {
+        Whitespace:                "#bbbbbb",
+
         Comment:                   "#888",
         Comment.Preproc:           "#579",
+        Comment.Special:           "bold #cc0000",
 
         Keyword:                   "bold #080",
         Keyword.Pseudo:            "#038",
